'''
Created on May 11, 2012

@author: william
'''

import numpy as np
import matplotlib.pyplot as plt


def smooth(x, window_len=11, window='hanning'):
    """smooth the data using a window with requested size.

    This method is based on the convolution of a scaled window with the signal.
    The signal is prepared by introducing reflected copies of the signal 
    (with the window size) in both ends so that transient parts are minimized
    in the begining and end part of the output signal.

    input:
        x: the input signal 
        window_len: the dimension of the smoothing window; should be an odd integer
        window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'
            flat window will produce a moving average smoothing.

    output:
        the smoothed signal

    example:

    t=linspace(-2,2,0.1)
    x=sin(t)+randn(len(t))*0.1
    y=smooth(x)

    see also: 

    numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve
    scipy.signal.lfilter

    """

    if x.ndim != 1:
        raise ValueError, "smooth only accepts 1 dimension arrays."

    if x.size < window_len:
        raise ValueError, "Input vector needs to be bigger than window size."

    if window_len < 3:
        return x

    if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError, "Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'"

    s = np.r_[x[window_len - 1:0:-1], x, x[-1:-window_len:-1]]
    # print(len(s))
    if window == 'flat':  # moving average
        w = np.ones(window_len, 'd')
    else:
        w = eval('numpy.' + window + '(window_len)')

    y = np.convolve(w / w.sum(), s, mode='valid')
    return y[(window_len / 2 - 1):-(window_len / 2)]


def plot_starlight_fit_single(sl_out, title='', i_fig=1, figsize=(10, 7), smooth_plot=False):
    plot_starlight_output(sl_out.spectra.l_obs, sl_out.spectra.f_obs, sl_out.spectra.f_syn, sl_out.spectra.f_wei,
                          sl_out.population.popage_base, sl_out.population.popZ_base, sl_out.population.popx,
                          sl_out.population.popmu_cor, sl_out.keywords['A_V'], sl_out.keywords['v_0'],
                          sl_out.keywords['v_d'], sl_out.keywords['SN_normwin'], sl_out.keywords['adev'],
                          sl_out.keywords['chi2'], title, i_fig, figsize, smooth_plot)


def plot_starlight_output(l_obs, f_obs, f_syn, f_wei, popage_base, popZ_base, popx, popmu_cor,
                          A_V, v_0, v_d, SN_normwin, adev, chi2,
                          title='', i_fig=1, figsize=(10, 7), smooth_plot=False):

    fig = plt.figure(i_fig, figsize=figsize)
    fig.clf()

    # Some configurations
    Zcolor = ['magenta', 'cyan', 'blue', 'green', 'black', 'red',
              'magenta', 'cyan', 'blue', 'green', 'black,', 'red']
    lim_res_low = -.29
    lim_res_upp = .99

    # Divide in two boxes 60% and 30% of size.
    box_left_low = [.1, .1, .4, .2]  # left, bottom, width, height
    box_left_upp = [.1, .3, .4, .6]
    box_right_low = [.6, .1, .3, .3]
    box_right_upp = [.6, .4, .3, .3]

    # Create the axes
    axleft_low = fig.add_axes(box_left_low)
    axleft_upp = fig.add_axes(box_left_upp, sharex=axleft_low)
    axright_low = fig.add_axes(box_right_low)
    axright_upp = fig.add_axes(box_right_upp, sharex=axright_low)

    #.... Plots ...
    # LEFT
    ### LEFT - UPPER
    if smooth_plot:
        axleft_upp.plot(
            l_obs, smooth(f_obs, smooth_plot), color='blue')  # Observed
        axleft_upp.plot(
            l_obs, smooth(f_syn, smooth_plot), color='red')  # Synthetic
        axleft_upp.plot(l_obs, f_obs, color='cyan', alpha=.3)  # Observed
        axleft_upp.plot(l_obs, f_syn, color='magenta', alpha=.3)  # Synthetic
    else:
        axleft_upp.plot(l_obs, f_obs, color='blue')  # Observed
        axleft_upp.plot(l_obs, f_syn, color='red')  # Synthetic

    np.seterr(divide='ignore')  # Ignore zero-division error.
    err = np.ma.array(np.divide(1., (f_wei)))

    if(smooth_plot):
        axleft_upp.plot(l_obs[err > 0], smooth(err[err > 0], smooth_plot), color='black')  # Error
        axleft_upp.plot(l_obs[err > 0], err[err > 0], color='gray', alpha=.3)  # Error
    else:
        axleft_upp.plot(l_obs[err > 0], err[err > 0], color='black')  # Error

    mi_ = np.min(l_obs)
    ma_ = np.max(l_obs)
    axleft_upp.set_xlim(mi_ * 0.9, ma_ * 1.1)

    ### LEFT - LOWER
    if smooth_plot:
        res = np.ma.array(smooth(f_obs, smooth_plot) - smooth(f_syn, smooth_plot))
    else:
        res = np.ma.array(f_obs - f_syn)
    axleft_low.plot(l_obs, res, color='black')  # All
    aux_ = np.ma.masked_where(f_wei < 0, res)
    axleft_low.plot(l_obs, aux_, color='blue')  # If wei > 0
    aux_ = np.ma.masked_where(f_wei != 0, res)
    # If wei == 0 -> Masked
    axleft_low.plot(l_obs, aux_, color='magenta')
    aux_ = np.ma.masked_where(f_wei != -1, res)
    # If wei == -1 -> Clipped
    axleft_low.plot(l_obs, aux_, 'x', color='red')
    aux_ = np.ma.masked_where(f_wei != -2, res)
    # If wei == -2 -> Flagged
    axleft_low.plot(l_obs, aux_, 'o', color='green')
    axleft_low.set_ylim(lim_res_low, lim_res_upp)

    # RIGHT
    log_age = np.log10(popage_base)
    Z = popZ_base
    xj = popx
    mu_cor = popmu_cor

    # Define bar spacing
    d = log_age[1:] - log_age[:-1]
    Zwidth = np.min(d[d > 0])

    ### RIGHT - UPPER
    aux_sum = np.zeros(np.shape(Z[Z == Z[0]]))
    i_color = 0
    for i_Z in np.unique(Z):
        v_ = (Z == i_Z)
        axright_upp.bar(log_age[v_], xj[v_], width=Zwidth, align='center', color=Zcolor[i_color],
                        bottom=aux_sum, label=('%3.4f' % i_Z))
        aux_sum = aux_sum + xj[v_]
        i_color = i_color + 1

    ### RIGHT - LOWER
    aux_sum = aux_sum * 0.
    i_color = 0
    for i_Z in np.unique(Z):
        v_ = (Z == i_Z)
        aux_sum = aux_sum + mu_cor[v_]
        i_color = i_color + 1
    axright_low.bar(log_age[v_], aux_sum, width=Zwidth,
                    align='center', color='white', label=('%3.4f' % i_Z), log=True)
    axright_low.set_xlim(np.min(log_age) * .99, np.max(log_age) * 1.01)

    # Remove last lower ytick
    axleft_low.set_yticks(axleft_low.get_yticks()[:-1])
    axright_low.set_yticks(axright_low.get_yticks()[:-1])

    # Axis Labels
    axright_low.set_xlabel('log age [yr]')
    axright_low.set_ylabel('$\mu_j$ [%]')
    axright_upp.set_ylabel('$x_j$ [%]')

    axleft_low.set_xlabel('$\lambda [\AA]$')
    axleft_low.set_ylabel('Residual spectrum')
    axleft_upp.set_ylabel('$F_\lambda [normalized]$')

    # Some fit labels...
    fig.text(.6, .88, '$\chi^2 =\ $' + ('%3.2f' % chi2))
    fig.text(.6, .84, 'adev = ' + ('%3.2f' % adev))
    if SN_normwin is not None:
        fig.text(.6, .80, '$S/N =\ $' + ('%3.2f' % SN_normwin))
    fig.text(.6, .76, '$A_V =\ $' + ('%3.2f' % A_V))
    fig.text(.6, .72, '$\sigma_\star =\ ' + ('%3.2f' % v_d) + '\ $km/s\t$v_\star =\ ' + ('%3.2f' % v_0) + '\ $km/s')
    fig.text(.6, .92, title, color='red')

    axright_upp.legend(loc=2, prop={'size': 8})
